
# Updates values slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' Generic Update Values
#' 
#' Generic function for updating fit values for betas. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta values.
setGeneric("updateValues<-", 
           function(object,value){standardGeneric("updateValues<-")})

# Updates standard errors slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' Generic Update Errors
#' 
#' Generic function for updating fit standard errors for betas. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta standard errors. 
setGeneric("updateErrors<-", 
           function(object,value){standardGeneric("updateErrors<-")})

# Updates z-scores slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' Generic Update Z-score function
#' 
#' Generic function for updating fit standard z-scores for betas. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta z-scores. 
setGeneric("updateZ<-", 
           function(object,value){standardGeneric("updateZ<-")})

# Updates significance slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' Generic Update Significance
#' 
#' Generic function for updating fit significance for betas. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta significance. 
setGeneric("updateSig<-", 
           function(object,value){standardGeneric("updateSig<-")})

# Get Generic Functions

# Returns fit values for betas. 
#' Generic function for returning values slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getValues", 
           function(object){standardGeneric("getValues")})

# Returns fit standard errors for betas. 
#' Generic function for returning standard errors slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getErrors", 
           function(object){standardGeneric("getErrors")})

# Returns fit z-scores for betas. 
#' Generic function for returning z-scores slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getZ", 
           function(object){standardGeneric("getZ")})

# Returns significance for fit betas. 
#' Generic function for returning significance slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getSig", 
           function(object){standardGeneric("getSig")})

# Returns old fit values for betas. 
#' Generic function for returning old values slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getOldValues", 
           function(object){standardGeneric("getOldValues")})

# Returns old fit standard errors for betas. 
#' Generic function for returning old errors slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getOldErrors", 
           function(object){standardGeneric("getOldErrors")})

# Returns old fit z-scores for betas. 
#' Generic function for returning old z-scores slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getOldZ", 
           function(object){standardGeneric("getOldZ")})

# Returns old fit significance for betas. 
#' Generic function for returning old significance slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getOldSig", 
           function(object){standardGeneric("getOldSig")})

#' Generic function for returning equivalence matrix 
#' 
#' Returns an equivalence matrix for an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. Checks for conflicting instructions and gets rid of redundancies.
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getEquivMat", 
           function(object){standardGeneric("getEquivMat")})

#' Generic function for returning model seed length 
#' 
#' Returns the seed length for an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getSeedLen", 
           function(object){standardGeneric("getSeedLen")})


#' Generic function for returning length (bp) of upstream footprint extension 
#' 
#' Returns the length (bp) of upstream footprint extension  for an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getUpFootprintExtend", 
           function(object){standardGeneric("getUpFootprintExtend")})

#' Generic function for returning length (bp) of upstream footprint extension for full feature set
#' 
#' Returns the length (bp) of upstream footprint extension for full feature set  for an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getFsUpFootprintExtend", 
           function(object){standardGeneric("getFsUpFootprintExtend")})

#' Generic function for returning length (bp) of downstream footprint extension 
#' 
#' Returns the length (bp) of downstream footprint extension  for an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getDownFootprintExtend", 
           function(object){standardGeneric("getDownFootprintExtend")})

#' Generic function for returning length (bp) of downstream footprint extension for full feature set
#' 
#' Returns the length (bp) of downstream footprint extension for full feature set  for an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getFsDownFootprintExtend", 
           function(object){standardGeneric("getFsDownFootprintExtend")})


#' Generic function for returning footprint length (bp) 
#' 
#' Returns fpLen slot from an object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getFpLen", 
           function(object){standardGeneric("getFpLen")})


#' Generic function for returning positions 
#'
#' Returns set slot from object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
setGeneric("getPositions", 
           function(object){standardGeneric("getPositions")})

#' Generic Number of Views per Strand
#'
#' Returns 'numViews' slot. 
#' 
#' @param object Object of class \linkS4class{model}.
setGeneric("getNumViews", 
           function(object){standardGeneric("getNumViews")})


#' Generic 'rcSymmetric'
#'
#' Returns'rcSymmetric' slot. 
#' 
#' @param object Object of class \linkS4class{model}.
setGeneric("getRcSymmetric", 
           function(object){standardGeneric("getRcSymmetric")})

#' Get Rounds
#'
#' Gets 'rounds' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getRounds", 
           function(object){standardGeneric("getRounds")})


#' Get Shape Parameters Used from \linkS4class{Shape}
#' 
#' Gets 'shapeParamsUsed' slot from object of class \linkS4class{Shape}.
#' 
#' @param object Object of class \linkS4class{Shape}.
setGeneric("getShapeParamsUsed", 
           function(object){standardGeneric("getShapeParamsUsed")})

#' Generic function for get name slot.
#'
#' Gets 'name' slot from object of class \linkS4class{model}.
#'
#' @param object object of class 'model'
setGeneric("getName", 
           function(object){standardGeneric("getName")})

#' Generic Function for Get Variable Region Length
#'
#' Gets 'varRegLen' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getVarRegLen", 
           function(object){standardGeneric("getVarRegLen")})

#' Generic function for Get Left Fixed Sequence
#'
#' Gets 'leftFixedSeq' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getLeftFixedSeq", 
           function(object){standardGeneric("getLeftFixedSeq")})

#' Generic function for Get Right Fixed Sequence
#'
#' Gets 'rightFixedSeq' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getRightFixedSeq", 
           function(object){standardGeneric("getRightFixedSeq")})

#' Generic function for Get Left Fixed Sequence Overlap
#'
#' Gets 'leftFixedSeqOverlap' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getLeftFixedSeqOverlap", 
           function(object){standardGeneric("getLeftFixedSeqOverlap")})

#' Generic function for Get Right Fixed Sequence Overlap
#'
#' Gets 'rightFixedSeqOverlap' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getRightFixedSeqOverlap", 
           function(object){standardGeneric("getRightFixedSeqOverlap")})

#' Generic Function for Getting Consensus Sequence 
#'
#' Gets 'consensusSeq' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getConsensusSeq", 
           function(object){standardGeneric("getConsensusSeq")})

#' Generic function for Getting Affinity Type
#'
#' Gets 'affinityType' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getAffinityType", 
           function(object){standardGeneric("getAffinityType")})

#' Generic function for Getting Confidence Level
#'
#' Gets 'confidenceLevel' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getConfidenceLevel", 
           function(object){standardGeneric("getConfidenceLevel")})

#' Generic function for Getting Minimum Affinity
#'
#' Gets 'minAffinity' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getMinAffinity", 
           function(object){standardGeneric("getMinAffinity")})

#' Generic function for Getting Missing Value Suppression Factor
#'
#' Gets 'missingValueSuppression' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getMissingValueSuppression", 
           function(object){standardGeneric("getMissingValueSuppression")})

#' Generic function for Getting Minimum Seed Value
#'
#' Gets 'minSeedValue' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getMinSeedValue", 
           function(object){standardGeneric("getMinSeedValue")})

#' Generic Function for Getting Regression Formula
#'
#' Gets 'regressionFormula' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getRegressionFormula", 
           function(object){standardGeneric("getRegressionFormula")})

#' Generic Function for Getting Iteration
#'
#' Gets 'iteration' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getIteration", 
           function(object){standardGeneric("getIteration")})

#' Generic function for Getting includeDNAstrand (logical)
#'
#' Gets 'includeDNAstrand' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getIncludeDNAstrand", 
           function(object){standardGeneric("getIncludeDNAstrand")})

#' Generic function for Getting includeView (logical)
#'
#' Gets 'includeView' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getIncludeView", 
           function(object){standardGeneric("getIncludeView")})


#' Generic function for getting includeShape (logical)
#'
#' Gets 'includeShape' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getIncludeShape", 
           function(object){standardGeneric("getIncludeShape")})

#' Generic Function for Getting useFixedValuesOffset.N (logical)
#'
#' Gets 'useFixedValuesOffset.N' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getUseFixedValuesOffset.N", 
           function(object){standardGeneric("getUseFixedValuesOffset.N")})

#' Generic Function for Getting useFixedValuesOffset.Shape (logical)
#'
#' Gets 'useFixedValuesOffset.Shape' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getUseFixedValuesOffset.Shape", 
           function(object){standardGeneric("getUseFixedValuesOffset.Shape")})

#' Generic Function for Getting Length of Extension Upstream Bases for Shape Parameters
#'
#' Gets 'exUpBases' slot from object of class \linkS4class{model}
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getExUpBases", 
           function(object){standardGeneric("getExUpBases")})

#' Generic Function for Getting Length of Extension Downstream Bases for Shape Parameters
#'
#' Gets 'exDownBases' slot from object of class \linkS4class{model}
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getExDownBases", 
           function(object){standardGeneric("getExDownBases")})

#' Generic Function for Getting Shape Parameters Used
#'
#' Gets 'shapeParamsUsed' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getShapeTable", 
           function(object){standardGeneric("getShapeTable")})

#' Generic Function for Getting Features
#'
#' Gets 'featureSet' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getFeatures", 
           function(object){standardGeneric("getFeatures")})

#' Generic Function for getting 'verbose' slot
#'
#' Gets 'verbose' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getVerbose", 
           function(object){standardGeneric("getVerbose")})

#' Generic Function for Getting Date Initialized
#'
#' Gets 'dateInitialized' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getDateInitialized", 
           function(object){standardGeneric("getDateInitialized")})

#' Generic function for Getting Date Last Modified
#'
#' Gets 'dateLastModified' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}.
setGeneric("getDateLastModified", 
           function(object){standardGeneric("getDateLastModified")})

#' Generic Function for Getting Date Betas Last Added.
#'
#' Gets 'dateBetasLastAdded' slot from object of class \linkS4class{model}.
#'
#' @param object Object of class \linkS4class{model}
setGeneric("getDateBetasLastAdded", 
           function(object){standardGeneric("getDateBetasLastAdded")})

#' Generic function for Getting 'GlmFits' for class \linkS4class{featureSet}
#' 
#' Gets 'GlmFits' slot from object of class \linkS4class{featureSet}.
#' 
#' @param object Object of class \linkS4class{featureSet}.
setGeneric("getGlmFits", 
           function(object){standardGeneric("getGlmFits")})

#' Generic function for Getting Design Matrix Summary for \linkS4class{featureSet}
#' 
#' Gets 'designMatrixSummary' slot from object of class \linkS4class{featureSet}.
#' 
#' @param object Object of class \linkS4class{featureSet}.
setGeneric("getDesignMatrixSummary", 
           function(object){standardGeneric("getDesignMatrixSummary")})

#' Get \linkS4class{N} for \linkS4class{featureSet} or \linkS4class{model}
#'
#' Gets \linkS4class{N} slot from object of class \linkS4class{featureSet} or \linkS4class{model}.
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
setGeneric("getN",
           function(object){standardGeneric("getN")})

#' Get \linkS4class{Intercept} for \linkS4class{featureSet} or \linkS4class{model}
#'
#' Gets \linkS4class{Intercept} slot from object of class \linkS4class{featureSet} or \linkS4class{model}.
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
setGeneric("getIntercept",
           function(object){standardGeneric("getIntercept")})

#' Get \linkS4class{Shape} for \linkS4class{featureSet} or \linkS4class{model}
#'
#' Gets \linkS4class{Shape} slot from object of class \linkS4class{featureSet} or \linkS4class{model}.
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
setGeneric("getShape",
           function(object){standardGeneric("getShape")})

#' Generic function for returning design matrix
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param design Data frame with design matrix, as generated by addDesignMatrix function.
setGeneric("getDesignMatrix",
           function(object, design){standardGeneric("getDesignMatrix")})


#' Generic function for returning feature design
#' 
#' @param object Object of class \linkS4class{model} or \linkS4class{featureSet}
#' @param ... Arguments to be passed to methods for specific classes
setGeneric("getFeatureDesign",
           function(object,...){standardGeneric("getFeatureDesign")})

#' Generic function for returning values for plotting
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Intercept}, or \linkS4class{Shape}
#' @param ... Arguments to be passed to methods for specific classes
setGeneric("getPlotValues",
           function(object,...){standardGeneric("getPlotValues")})

#' Generic function for returning PSAM
#' 
#' @param object Object of class \linkS4class{model} or \linkS4class{N}
#' @param ... Arguments to be passed to methods for specific classes
setGeneric("getPSAM",
           function(object, ...){standardGeneric("getPSAM")})

#' Generic function for returning values for plotting
#' 
#' @param object Object of class \linkS4class{N}
#' @param ... Arguments to be passed to methods for specific classes
setGeneric("getTopSeqLabels",
           function(object, ...){standardGeneric("getTopSeqLabels")})




#' Generic function for setting values slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta values.
setGeneric("setValues<-", 
           function(object,value){standardGeneric("setValues<-")})

#' Generic function for setting errors slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta standard errors
setGeneric("setErrors<-", 
           function(object,value){standardGeneric("setErrors<-")})

#' Generic function for setting z slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta z-scores
setGeneric("setZ<-", 
           function(object,value){standardGeneric("setZ<-")})

#' Generic function for setting significance slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Matrix containing beta significance
setGeneric("setSig<-", 
           function(object,value){standardGeneric("setSig<-")})

#' Generic function for setting 'oldValues' slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Array containing old beta values.
setGeneric("setOldValues<-", 
           function(object,value){standardGeneric("setOldValues<-")})

#' Generic function for setting 'oldErrors' slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Array containing old beta standard errors.
setGeneric("setOldErrors<-", 
           function(object,value){standardGeneric("setOldErrors<-")})

#' Generic function for setting 'oldZ' slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Array containing old z-scores.
setGeneric("setOldZ<-", 
           function(object,value){standardGeneric("setOldZ<-")})

#' Generic function for setting 'oldSig' slot in featureSet and its feature-type specific slots (e.g. N, Shape, etc.)
#' 
#' @param object Object of class \linkS4class{model}, \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{Intercept}.
#' @param value Array containing old significance. 
setGeneric("setOldSig<-", 
           function(object,value){standardGeneric("setOldSig<-")})

#' Generic function for setting 'equivMat' slot 
#' 
#' @param object Object of class \linkS4class{N} or \linkS4class{Shape}.
#' @param value Output of formatEquivalenceMatrix
setGeneric("setEquivMat<-", 
           function(object,value){standardGeneric("setEquivMat<-")})

#' Generic function for setting 'seedLen' slot 
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape},\linkS4class{model}, or \linkS4class{featureSet}.
#' @param value seed length of the full model
setGeneric("setSeedLen<-", 
           function(object,value){standardGeneric("setSeedLen<-")})


#' Generic function for setting length (bp) of upstream footprint extension 
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape},\linkS4class{model}, or \linkS4class{featureSet}.
#' @param value length (bp) of upstream footprint extension 
setGeneric("setUpFootprintExtend<-", 
           function(object,value){standardGeneric("setUpFootprintExtend<-")})

#' Generic function for setting length (bp) of upstream footprint extension for full feature set
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape},\linkS4class{model}, or \linkS4class{featureSet}.
#' @param value length (bp) of upstream footprint extension for full feature set
setGeneric("setFsUpFootprintExtend<-", 
           function(object,value){standardGeneric("setFsUpFootprintExtend<-")})

#' Generic function for setting length (bp) of downstream footprint extension 
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape},\linkS4class{model}, or \linkS4class{featureSet}.
#' @param value length (bp) of downstream footprint extension 
setGeneric("setDownFootprintExtend<-", 
           function(object,value){standardGeneric("setDownFootprintExtend<-")})

#' Generic function for setting length (bp) of downstream footprint extension for full feature set
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape},\linkS4class{model}, or \linkS4class{featureSet}.
#' @param value length (bp) of downstream footprint extension for full feature set
setGeneric("setFsDownFootprintExtend<-", 
           function(object,value){standardGeneric("setFsDownFootprintExtend<-")})

#' Generic function for setting 'fpLen'. Note that this function leads to an error as 'fpLen' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape}, or \linkS4class{model}
#' @param value footprint length (bp)
setGeneric("setFpLen<-", 
           function(object,value){standardGeneric("setFpLen<-")})

#' Generic function for setting number of positions in  \linkS4class{N} or \linkS4class{Shape}
#' 
#' @param object Object of class \linkS4class{N} or \linkS4class{Shape}
#' @param value Vector of positions within \linkS4class{featureSet} 
setGeneric("setPositions<-", 
           function(object,value){standardGeneric("setPositions<-")})

#' Generic function for setting 'numViews'. 
#' 
#' Note that this function leads to an error as 'numViews' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{featureSet}, or \linkS4class{model}
#' @param value number of views along DNA strand for model with fpLen
setGeneric("setNumViews<-", 
           function(object,value){standardGeneric("setNumViews<-")})

#' Generic function for setting 'rcSymmetric'. 
#' 
#' @param object Object of class \linkS4class{featureSet}, or \linkS4class{model}
#' @param value logical: should reverse complement symmetry be imposed on the model?
setGeneric("setRcSymmetric<-", 
           function(object,value){standardGeneric("setRcSymmetric<-")})

#' Generic function for setting 'rounds'. 
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
#' @param value 'rounds' slot must is a list of length one, where the list has at least one entry
setGeneric("setRounds<-", 
           function(object,value){standardGeneric("setRounds<-")})


#' Generic function for setting 'shapeParamsUsed'. 
#' 
#' @param object Object of class \linkS4class{featureSet}, \linkS4class{model} or  \linkS4class{Shape}.
#' @param value 'shapeParamsUsed' slot must be a list of length one drawn from ('HelT', 'MGW', 'ProT', 'Roll')
setGeneric("setShapeParamsUsed<-", 
           function(object,value){standardGeneric("setShapeParamsUsed<-")})

#' Generic function for setting 'name'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: 'name' of model
setGeneric("setName<-", 
           function(object,value){standardGeneric("setName<-")})

#' Generic function for setting 'varRegLen'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: 'varRegLen' is the length of the variable region for the SELEX probe
setGeneric("setVarRegLen<-", 
           function(object,value){standardGeneric("setVarRegLen<-")})

#' Generic function for setting 'leftFixedSeq'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: Left fixed sequence for SELEX probe
setGeneric("setLeftFixedSeq<-", 
           function(object,value){standardGeneric("setLeftFixedSeq<-")})

#' Generic function for setting 'rightFixedSeq'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: Right fixed sequence for SELEX probe
setGeneric("setRightFixedSeq<-", 
           function(object,value){standardGeneric("setRightFixedSeq<-")})

#' Generic function for setting 'leftFixedSeqOverlap'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: Overlap of fit window and left fixed sequence for SELEX probe
setGeneric("setLeftFixedSeqOverlap<-", 
           function(object,value){standardGeneric("setLeftFixedSeqOverlap<-")})

#' Generic function for setting 'rightFixedSeqOverlap'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: Overlap of fit window and right fixed sequence for SELEX probe
setGeneric("setRightFixedSeqOverlap<-", 
           function(object,value){standardGeneric("setRightFixedSeqOverlap<-")})

#' Generic function for setting 'consensusSeq'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: Consensus Sequence to seed initial model
setGeneric("setConsensusSeq<-", 
           function(object,value){standardGeneric("setConsensusSeq<-")})

#' Generic function for setting 'affinityType'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: Affinity Type to use in fitting model
setGeneric("setAffinityType<-", 
           function(object,value){standardGeneric("setAffinityType<-")})

#' Generic function for setting 'confidenceLevel'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: Consensus Sequence to seed initial model
setGeneric("setConfidenceLevel<-", 
           function(object,value){standardGeneric("setConfidenceLevel<-")})

#' Generic function for setting 'minAffinity'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: Minimum Affinity (0 <= minAffinity < 1)
setGeneric("setMinAffinity<-", 
           function(object,value){standardGeneric("setMinAffinity<-")})

#' Generic function for setting 'missingValueSuppression'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: ddG suppression of features not occurring in the design dataframe relative to the lowest value betas that are fit. 
setGeneric("setMissingValueSuppression<-", 
           function(object,value){standardGeneric("setMissingValueSuppression<-")})

#' Generic function for setting 'minSeedValue'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: minimum relative affinity allowed in seeding psam 
setGeneric("setMinSeedValue<-", 
           function(object,value){standardGeneric("setMinSeedValue<-")})

#' Generic function for setting 'regressionFormula'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: regression model to be used in fitting
setGeneric("setRegressionFormula<-", 
           function(object,value){standardGeneric("setRegressionFormula<-")})

#' Generic function for setting 'iteration'. 
#' 
#' Note that this function leads to an error as 'iteration' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric
setGeneric("setIteration<-", 
           function(object,value){standardGeneric("setIteration<-")})

#' Generic function for setting 'indludeDNAstrand'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value logical: should the DNA strand containing the max affinity view be included as a fitting parameter?
setGeneric("setIncludeDNAstrand<-", 
           function(object,value){standardGeneric("setIncludeDNAstrand<-")})

#' Generic function for setting 'includeView'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value logical: should position of the max affinity view be included as a fitting parameter?
setGeneric("setIncludeView<-", 
           function(object,value){standardGeneric("setIncludeView<-")})

#' Generic function for setting 'includeShape'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value logical: should shape parameters be included as fitting parameters?
setGeneric("setIncludeShape<-", 
           function(object,value){standardGeneric("setIncludeShape<-")})

#' Generic function for setting 'useFixedValuesOffset.N'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value logical: 'useFixedValuesOffset.N'' slot for \linkS4class{model} object.
setGeneric("setUseFixedValuesOffset.N<-", 
           function(object,value){standardGeneric("setUseFixedValuesOffset.N<-")})

#' Generic function for setting 'useFixedValuesOffset.Shape'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value logical: 'useFixedValuesOffset.Shape'' slot for \linkS4class{model} object.
setGeneric("setUseFixedValuesOffset.Shape<-", 
           function(object,value){standardGeneric("setUseFixedValuesOffset.Shape<-")})

#' Generic function for setting 'exUpBases'. 
#' 
#' Note that this function leads to an error as 'exUpBases' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: 'exUpBases' slot for 'model' object
setGeneric("setExUpBases<-", 
           function(object,value){standardGeneric("setExUpBases<-")})

#' Generic function for setting 'exDownBases'. 
#' 
#' Note that this function leads to an error as 'exDownBases' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value numeric: 'exDownBases' slot for 'model' object
setGeneric("setExDownBases<-", 
           function(object,value){standardGeneric("setExDownBases<-")})

#' Generic function for setting 'shapeTable'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value 'shapeTable' slot for \linkS4class{model} object.
setGeneric("setShapeTable<-", 
           function(object,value){standardGeneric("setShapeTable<-")})

#' Generic function for setting 'featureSet'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value featureSet: 'features' slot for \linkS4class{model} object
setGeneric("setFeatures<-", 
           function(object,value){standardGeneric("setFeatures<-")})

#' Generic function for setting 'verbose'. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value logical
setGeneric("setVerbose<-", 
           function(object,value){standardGeneric("setVerbose<-")})

#' Generic function for setting 'dateInitialized'. 
#' 
#' Note that this function leads to an error as 'dateInitialized' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: 'dateInitialized' slot for 'model' object
setGeneric("setDateInitialized<-", 
           function(object,value){standardGeneric("setDateInitialized<-")})

#' Generic function for setting 'dateLastModified'. 
#' 
#' Note that this function leads to an error as 'dateLastModified' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: 'dateLastModified' slot for 'model' object
setGeneric("setDateLastModified<-", 
           function(object,value){standardGeneric("setDateLastModified<-")})

#' Generic function for setting 'dateBetasLastAdded'. 
#' 
#' Note that this function leads to an error as 'dateBetasLastAdded' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{model}.
#' @param value character: 'dateBetasLastAdded' slot for 'model' object
setGeneric("setDateBetasLastAdded<-", 
           function(object,value){standardGeneric("setDateBetasLastAdded<-")})

#' Generic function for setting 'glmFits'. 
#' 
#' Note that this function leads to an error as 'glmFits' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{featureSet}.
#' @param value  'glmFits' slot for 'featureSet' object
setGeneric("setGlmFits<-", 
           function(object,value){standardGeneric("setGlmFits<-")})

#' Generic function for setting 'designMatrixSummary'. 
#' 
#' Note that this function leads to an error as 'designMatrixSummary' should not be manually set. 
#' 
#' @param object Object of class \linkS4class{featureSet}.
#' @param value  designMatrixSummary 
setGeneric("setDesignMatrixSummary<-", 
           function(object,value){standardGeneric("setDesignMatrixSummary<-")})

#' Generic function for  finalizing beta values'. 
#' 
#' Finalizes beta values in \linkS4class{model} object by restoring NA's that were removed to allow for the iteration.
#' 
#' @param object Object of class \linkS4class{featureSet}.
setGeneric("finalizeFeatureBetas",
            function(object){standardGeneric("finalizeFeatureBetas")})

#' Generic function for setting \linkS4class{N}. 
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
#' @param value N: Object of class \linkS4class{N}.
setGeneric("setN<-",
           function(object,value){standardGeneric("setN<-")})

#' Generic function for setting \linkS4class{Intercept}. 
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
#' @param value Intercept: Object of class \linkS4class{Intercept}.
setGeneric("setIntercept<-",
           function(object,value){standardGeneric("setIntercept<-")})

#' Generic function for setting \linkS4class{Shape}. 
#' 
#' @param object Object of class \linkS4class{featureSet} or \linkS4class{model}.
#' @param value Shape: Object of class \linkS4class{Shape}.
setGeneric("setShape<-",
           function(object,value){standardGeneric("setShape<-")})

#' Generic function for building equivalence matrix encoding reverse complement symmetry
#' 
#' @param object Object of class \linkS4class{N} or \linkS4class{Shape}.
setGeneric("buildSymmetricEquivalenceMatrix",
           function(object){standardGeneric("buildSymmetricEquivalenceMatrix")})

#' Generic function for building equivalence matrix encoding no symmetries 
#' 
#' @param object Object of class \linkS4class{N} or \linkS4class{Shape}.
setGeneric("buildNullEquivalenceMatrix",
           function(object){standardGeneric("buildNullEquivalenceMatrix")})

#'Generic function for formatting an equivalence matrix checking for conflicting instructions and getting rid of redundancies.
#' 
#' @param object Object of class \linkS4class{N} or \linkS4class{Shape}.
setGeneric("formatEquivalenceMatrix",
           function(object){standardGeneric("formatEquivalenceMatrix")})



#' Generic Function for adding a seeding nucleotide PSAM
#' 
#' @param object Object of class \linkS4class{model}. 
#' @param value Seeding PSAM for nucleotides.
setGeneric("addSeedPsam<-",
           function(object, value){standardGeneric("addSeedPsam<-")})

 
#' Generic Function for building seeding nucleotide PSAM
#' 
#' Uses k-mer table to build PSAM matrix for seeding SelexGLM.
#'
#' @param object Object of class \linkS4class{model}.
#' @param seedTable Kmer table used to build nucleotide seed. 
#' @return Kmer table-derived nucleotide Beta Values.
setGeneric (
  name = "seedTable2psam",
  def = function(object, seedTable){standardGeneric("seedTable2psam")}
)


#' Generic function for adding new betas
#' 
#' Adds new beta values using glm output. 
#' 
#' @name addNewBetas
#' @docType methods
#' @rdname addNewBetas-methods
#' @param object Object of class \linkS4class{N}, \linkS4class{Shape}, \linkS4class{Intercept}, or \linkS4class{model}.
#' @param design Design table used to fit glm model. 
#' @param value Glm fit object. 
#' @param ... Arguments to be passed to methods for specific classes
setGeneric("addNewBetas",
           function(object, design, value,...){standardGeneric("addNewBetas")})

#' Generic function for plotting Z-scores
#' 
#' @param object Object of class \linkS4class{Shape}.
#' @param ... Arguments to be passed to methods for specific classes
setGeneric("plotZs",
           function(object, ...){standardGeneric("plotZs")})

