% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model-class.R
\docType{methods}
\name{initialize,model-method}
\alias{initialize,model-method}
\title{Initialize object of class 'model'}
\usage{
\S4method{initialize}{model}(.Object, name, varRegLen, leftFixedSeq,
  rightFixedSeq, seedLen, consensusSeq, leftFixedSeqOverlap,
  rightFixedSeqOverlap, affinityType, numViews, rounds, confidenceLevel,
  minAffinity, missingValueSuppression, minSeedValue, regressionFormula,
  iteration, upFootprintExtend, downFootprintExtend, rcSymmetric,
  includeDNAstrand, includeView, includeShape, useFixedValuesOffset.N,
  useFixedValuesOffset.Shape, exUpBases, exDownBases, fpLen,
  shapeParamsUsed, shapeTable, features, verbose, dateInitialized,
  dateBetasLastAdded, dateLastModified, N, Intercept, Shape,
  N.upFootprintExtend, N.downFootprintExtend, N.set, N.values, N.errors,
  N.z, N.sig, N.oldValues, N.oldErrors, N.oldZ, N.oldSig, N.equivMat,
  I.values, I.errors, I.z, I.sig, I.oldValues, I.oldErrors, I.oldZ,
  I.oldSig, Shape.upFootprintExtend, Shape.downFootprintExtend, Shape.set,
  Shape.values, Shape.errors, Shape.z, Shape.sig, Shape.oldValues,
  Shape.oldErrors, Shape.oldZ, Shape.oldSig, Shape.equivMat, ...)
}
\arguments{
\item{.Object}{An object: see the "Initialize Methods" section.}

\item{name}{Name to be associated with the model object. If unspecified, empty string is assigned by default.}

\item{varRegLen}{Length of the library design for the selex data.}

\item{leftFixedSeq}{Fixed sequence to the left of random library in library design.}

\item{rightFixedSeq}{Fixed sequence to the right of random library in library design.}

\item{seedLen}{Length of k-mer used to seed the regression. If unspecified, empty numeric value is assigned by default.}

\item{consensusSeq}{Optional regular expression input specifying consensus sequence to be used to determine the seed sequence for building a PSAM from the k-mer table used to seed the analysis. If "consensusSeq" is input in iupac notation, it is converted to a standard regular expression during initialization of model object.}

\item{leftFixedSeqOverlap}{Number of bp into the left fixed flank to be considered in analysis. If unspecified, 0 is assigned by default.}

\item{rightFixedSeqOverlap}{Number of bp into the right fixed flank to be considered in analysis. If unspecified, \code{rightFixedSeqOverlap} is set to \code{leftFixedSeqOverlap} by default.}

\item{affinityType}{name of affinity variable in k-mer table to be used in building the seeding PSAM. If no value is specified, affinityType = 'Affinity'.}

\item{numViews}{number of distinct views of length k contained in each strand. Must be equal to leftFixedSeqOverlap+varRegLen+rightFixedSeqOverlap.}

\item{rounds}{list of length 1 with rounds[[1]] equal to a vector containing the rounds of data to be fit.}

\item{confidenceLevel}{view confidence level cutoff required for probes used in glm fit.}

\item{minAffinity}{minimum view affinity required for probes used in glm fit.}

\item{missingValueSuppression}{difference in ddG between the lowest fit beta values and estimates of betas for features that do not occur in the design matrix passed to glm fit.}

\item{minSeedValue}{minimum value for mono-nucleotide affinities specified in initial seeding PSAM. If no value is specified explicitly, minSeedValue = 0.}

\item{regressionFormula}{regression formula ('character' class) specifying the feature design of the 'model' object.}

\item{iteration}{number of glm iterations performed.}

\item{upFootprintExtend}{number of upstream base pairs to consider beyond the seed.}

\item{downFootprintExtend}{number of downstream base pairs to consider beyond the seed.}

\item{rcSymmetric}{boolean indicating whether of not a reverse complement symmetric model should be fit.}

\item{includeDNAstrand}{boolean indicating whether or not a strand orientation specific offset should be included in the 'model' fit. If includeView == TRUE, includeDNAstrand is ignored.}

\item{includeView}{boolean indicating whether or not a view+strand orientation specific offset should be included in the 'model' fit. If includeView == TRUE and rcSymmetric == TRUE, view offsets will be fit for only one of the two DNA strand orientations.}

\item{includeShape}{boolean indicating whether or not shape coefficients should be included in the 'model' fit.}

\item{useFixedValuesOffset.N}{logical: use initial, fixed values for mononucleotide positions within the model footprint, but not in N.set, as an offset to glm fit.}

\item{useFixedValuesOffset.Shape}{logical: use initial, fixed values for shape positions within the model footprint, but not in Shape.set, as an offset to glm fit.}

\item{exUpBases}{number of bases upstream of the mono-nucleotide position required to describe shape at the same position. If no value is specified explicitly, exUpBases = 2.}

\item{exDownBases}{number of bases downstream of the mono-nucleotide position required to describe shape at the same position. If no value is specified explicitly, exDownBases = 2.}

\item{fpLen}{footprint length of model. Must be fpLen = upFootprintExtend+seedLen+downFootprintExtend.}

\item{shapeParamsUsed}{list of length one contaning a vector of shape features to be included in 'model'. Only used if includeShape == TRUE.}

\item{shapeTable}{table of shape parameters to be used as predictors. Only used if includeShape == TRUE.}

\item{features}{object of class 'featureSet' containing specifics of model design and values for model parameters.}

\item{verbose}{logical: indicates whether to use verbose options in \pkg{SelexGLM} functions.}

\item{dateInitialized}{Character string generated by date() at the time when the object of class 'model' is created.}

\item{dateBetasLastAdded}{Character string generated by date() at the time when the object of class 'model' was last changed to incorporate new beta values.}

\item{dateLastModified}{Character string generated by date() at the time when the object of class 'model' was changed using any of the methods associated with it.}

\item{N}{object of class 'N'}

\item{Intercept}{object of class 'Intercept'}

\item{Shape}{object of class 'Shape'}

\item{N.upFootprintExtend}{number of upstream nucleotides to fit beyond the seed.}

\item{N.downFootprintExtend}{number of downstream nucleotides to fit beyond the seed.}

\item{N.set}{set of positions for which mononucleotides are fit where positions are given relative to the full feature set footprint.}

\item{N.values}{beta values for positions in N.set.}

\item{N.errors}{beta value errors for positions in N.set.}

\item{N.z}{beta value z scores for positions in N.set.}

\item{N.sig}{beta value significance for positions in N.set.}

\item{N.oldValues}{previous iteration beta values for positions in N.set.}

\item{N.oldErrors}{previous iteration beta values' errors for positions in N.set.}

\item{N.oldZ}{previous iteration beta values' z scores for positions in N.set.}

\item{N.oldSig}{previous iteration beta values' significance for positions in N.set.}

\item{N.equivMat}{matrix encoding reverse complement symmetries between positions with -1 and identical beta values shared between positions with 1.}

\item{I.values}{3-D array of beta values with entries corresponding to (Strand)x(View)x(Round) intercept values.}

\item{I.errors}{3-D array of errors for I.values.}

\item{I.z}{3-D array of Z-scores for I.values.}

\item{I.sig}{3-D array of p-values for I.values.}

\item{I.oldValues}{4-D array for storing previous iteration values beta values for I.values where each entry corresponds to (Strand)x(View)x(Round)x(Iteration).}

\item{I.oldErrors}{4-D array of previous iteration I.errors.}

\item{I.oldZ}{4-D array of previous iteration I.z.}

\item{I.oldSig}{4-D array of previous iteration I.sig.}

\item{Shape.upFootprintExtend}{number of upstream positions to fit shape features beyond the seed.}

\item{Shape.downFootprintExtend}{number of downstream positions to fit shape features beyond the seed.}

\item{Shape.set}{set of positions for which shape coefficients are fit where positions are given relative to the full feature set footprint.}

\item{Shape.values}{beta values for positions in Shape.set.}

\item{Shape.errors}{beta value errors for positions in Shape.set.}

\item{Shape.z}{beta value z scores for positions in Shape.set.}

\item{Shape.sig}{beta value significance for positions in Shape.set.}

\item{Shape.oldValues}{previous iteration beta values for positions in Shape.set.}

\item{Shape.oldErrors}{previous iteration beta values' errors for positions in Shape.set.}

\item{Shape.oldZ}{previous iteration beta values' z scores for positions in Shape.set.}

\item{Shape.oldSig}{previous iteration beta values' significance for positions in Shape.set.}

\item{Shape.equivMat}{matrix encoding reverse complement symmetries between positions with -1 at Shape.equivMat[position1, position2] entry. All other entries are 0.}

\item{...}{Arguments to be passed to methods}
}
\description{
Initializes an S4 class to represent full model features
}
